
<nz-card class="status-card" *ngIf="statusData$ | async as item">
    <nz-descriptions [nzColumn]="1" nzBordered>
        <nz-descriptions-item nzTitle="Status">
            <nz-tag [nzColor]="item?.rawValueQueueStatus ? 'success' : 'error'">
                {{ item?.rawValueQueueStatus ? 'Running' : 'Stopped' }}
            </nz-tag>
        </nz-descriptions-item>

        <nz-descriptions-item nzTitle="Queue Size">
            {{ item?.rawValueQueueSize }}
        </nz-descriptions-item>

        <nz-descriptions-item nzTitle="Total Processed">
            {{ item?.rawValueProcessedTotal }}
        </nz-descriptions-item>

        <nz-descriptions-item nzTitle="Last Error">
            <span class="error-text">{{ item?.rawValueLastError || 'None' }}</span>
        </nz-descriptions-item>
    </nz-descriptions>
</nz-card>